/*********************************************************************
 *                
 * Filename:      obex_main.h
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Dag Brattli <dagb@cs.uit.no>
 * Created at:    Mon Jul 20 22:28:23 1998
 * Modified at:   Fri Nov 19 16:02:18 1999
 * Modified by:   Pontus Fuchs <pontus@tactel.se>
 * 
 *     Copyright (c) 1998, 1999, Dag Brattli, All Rights Reserved.
 *      
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2 of the License, or (at your option) any later version.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *     MA  02111-1307  USA
 *     
 ********************************************************************/

#ifndef OBEX_MAIN_H
#define OBEX_MAIN_H

#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <linux/types.h>

/* Forward decl */
struct obex;
typedef struct obex obex_t;

#include "obex_object.h"
#include "obex_transport.h"
#include "netbuf.h"

/* use 0 for production, 1 for sendbuff, 2 for receivebuff and 3 for both */
#ifndef DEBUG_DUMPBUFFERS
#define DEBUG_DUMPBUFFERS 3
#endif

/* use 0 for production, 1 for verification, >2 for debug */
#ifndef NET_DEBUG
#define NET_DEBUG 4
#endif
#ifdef NET_DEBUG
unsigned int obex_net_debug;
#define DEBUG(n, args...) if (obex_net_debug >= (n)) g_print(args)
#else
#define DEBUG(n, args...)
#endif /* NET_DEBUG */

#define	OBEX_CLIENT		0
#define	OBEX_SERVER		1

#define OBEX_EV_PROGRESS	0	/* Progress has been made */
#define OBEX_EV_REQHINT		1
#define OBEX_EV_REQ		2
#define OBEX_EV_REQDONE		3	/* Request has finished */
#define OBEX_EV_LINKERR		4	/* Link has been disconnected */
#define OBEX_EV_PARSEERR	5	/* Malformed data encountered */

#define OBEX_VERSION		0x11      /* Version 1.1 */
#define OBEX_DEFAULT_MTU	1024
#define OBEX_MINIMUM_MTU	255      

#define OBEX_CMD	1
#define OBEX_RSP	0

#define OBEX_MAXINSTANCE 16

/* For OBEX_Init() */
#define OBEX_FL_ASYNC		0x01

#define OBEX_TRANS_IRDA		1
#define OBEX_TRANS_INET		2
#define OBEX_TRANS_CUST		3

/* For OBEX_ObejectAddHeader */
#define OBEX_FIT_ONE_PACKET	1

/* Standard headers */
#define OBEX_HEADER_COUNT	0xc0 /* Number of objects (used by connect) */
#define OBEX_HEADER_NAME	0x01 /* Name of the object */
#define OBEX_HEADER_TYPE	0x42 /* Type of the object */
#define OBEX_HEADER_TIME	0x44 /* Last modification of object */
#define OBEX_HEADER_TIME2	0xC4 /* This value is also allowed for time */
#define OBEX_HEADER_LENGTH	0xc3 /* Total lenght of object */
#define OBEX_HEADER_DESCRIPTION	0x05 /* Description of object */
#define OBEX_HEADER_TARGET	0x46 /* Identifies the target for the object */
#define OBEX_HEADER_BODY	0x48 /* Data part of the object */
#define OBEX_HEADER_BODY_END	0x49 /* Last data part of the object */
#define OBEX_HEADER_WHO		0x4a /* Identifies the sender of the object */
#define OBEX_HEADER_CONNECTION	0xcb /* Connection identifier */

/* Commands */
#define OBEX_CMD_CONNECT	0x00
#define OBEX_CMD_DISCONNECT	0x01
#define OBEX_CMD_PUT		0x02
#define OBEX_CMD_GET		0x03
#define OBEX_CMD_COMMAND	0x04
#define OBEX_CMD_SETPATH	0x05
#define OBEX_CMD_ABORT		0x7f
#define OBEX_FINAL		0x80

/* Responses */
#define	OBEX_RSP_CONTINUE		0x10
#define OBEX_RSP_SWITCH_PRO		0x11
#define OBEX_RSP_SUCCESS		0x20
#define OBEX_RSP_CREATED		0x21
#define OBEX_RSP_ACCEPTED		0x22
#define OBEX_RSP_NO_CONTENT		0x24
#define OBEX_RSP_BAD_REQUEST		0x40
#define OBEX_RSP_UNAUTHORIZED		0x41
#define OBEX_RSP_PAYMENT_REQUIRED	0x42
#define OBEX_RSP_FORBIDDEN		0x43
#define OBEX_RSP_NOT_FOUND		0x44
#define OBEX_RSP_METHOD_NOT_ALLOWED	0x45
#define OBEX_RSP_CONFLICT		0x49
#define OBEX_RSP_INTERNAL_SERVER_ERROR	0x50
#define OBEX_RSP_NOT_IMPLEMENTED	0x51
#define OBEX_RSP_DATABASE_FULL		0x60
#define OBEX_RSP_DATABASE_LOCKED	0x61

typedef void (*obex_event_t)(obex_object_t *obj, gint mode, gint event, gint obex_cmd, gint obex_rsp);

typedef struct {
	gint (*connect)(obex_t *handle);
	gint (*disconnect)(obex_t *handle);
	gint (*listen)(obex_t *handle);
	gint (*write)(obex_t *handle, guint8 *buf, gint buflen);
} obex_ctrans_t;

struct obex {
	guint16 mtu_tx;			/* Maximum OBEX Tx packet size */
        guint16 mtu_rx;			/* Maximum OBEX Rx packet size */

	gint fd;			/* Socket descriptor */

	gboolean async;

	GNetBuf *tx_msg;		/* Reusable transmit message */
	GNetBuf *rx_msg;		/* Reusable receive message */

	obex_object_t	*object;	/* Current object being transfered */      
	obex_event_t	eventcb;	/* Event-callback */

	volatile int response_next;	/* Next indication a response? */
	gint lastcmd;

	obex_transport_t trans;		/* Transport being used */
	obex_ctrans_t ctrans;
};

gint obex_register_async(obex_t *self, gint fd);
gint obex_create_socket(obex_t *self, gint domain, gboolean async);
gint obex_delete_socket(obex_t *self);

void obex_deliver_event(obex_t *self, gint mode, gint event, gint cmd, gint rsp, gboolean del);
void obex_input_handler(int signal);
gint obex_data_indication(obex_t *self, guint8 *buf, gint buflen);

gint obex_data_request(obex_t *self, GNetBuf *msg, gint opcode, 
		       gint response);

void obex_response_request(obex_t *self, guint8 opcode);

GString *obex_get_response_message(obex_t *self, gint rsp);

gint obex_transport_connect_request(obex_t *self);
void obex_transport_disconnect_request(obex_t *self);
gint obex_transport_listen(obex_t *self, char *service);
gint obex_transport_write(obex_t *self, GNetBuf *msg);
gint obex_transport_read(obex_t *self, guint8 *buf, gint buflen);

extern obex_t *async_self[OBEX_MAXINSTANCE];

#endif
